<?php
//
// Copyright 2015-2016, Julian Catchen <jcatchen@illinois.edu>
//
// This file is part of Chromonomer.
//
// Chromonomer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Chromonomer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Chromonomer.  If not, see <http://www.gnu.org/licenses/>.
//
include("header.php");

$ver = isset($_GET['v'])  ? $_GET['v']  : "";

if (strlen($ver) == 0)
    write_index_list();
else
    write_index($ver);

function write_index_list() {
    global $file_path;

    write_header("Chromonomer");

    //
    // Read the list of directories, we assume each directory contains output from a Chromonomer run.
    //
    $files = scandir($file_path);
    $versions = array();

    foreach ($files as $file) {
        if ($file == "images" || $file[0] == '.')
            continue;

        if (is_dir("$file_path/$file"))
            array_push($versions, $file);
    }

    print 
        "<h1 class=\"head\">Chromonomer Map/Assembly Integrations</h1>\n" .
        "<div class=\"version\">\n" .
        "<ol>\n";

    foreach ($versions as $ver) {
        $enc = urlencode($ver);

        //
        // Test if a description is available.
        //
        $desc = "";
        $path = $file_path  . "/" . $ver . "/chromonomer_description.log";

        if (file_exists($path)) {
            $desc = file_get_contents($path);
            $desc = rtrim($desc);
        }
        if (strlen($desc) > 0) 
            print "  <li><a href=\"index.php?v=$enc\">$ver</a> [<span class=\"chrrdesc\">$desc</span>]</li>\n";
        else
            print "  <li><a href=\"index.php?v=$enc\">$ver</a></li>\n";            
    }

    print 
        "</ol>\n" .
        "</div>\n";
}

function write_index($ver) {
    global $file_path, $root_path;

    write_header("Chromonomer: $ver");

    //
    // Read the list of linkage groups.
    //
    $files = scandir("$file_path/$ver");
    $lgs   = array();
    $genome_agp   = "";
    $scaffold_agp = "";
    $fasta        = "";
    $gene_ann     = "";
  
    foreach ($files as $file) {
        $parts = explode("_", $file);

        if (count($parts) > 1 && $parts[1] == "before.php")
            array_push($lgs, $parts[0]);
        else if (count($parts) > 1 && $parts[1] == "genome.agp")
            $genome_agp = $file;
        else if (count($parts) > 1 && $parts[1] == "scaffolds.agp")
            $scaffold_agp = $file;
        else if (count($parts) > 1 && $parts[1] == "integrated.fa")
            $fasta = $file;
        else if (substr($file, -15) == "-translated.gtf")
            $gene_ann = $file;
        else if (substr($file, -15)  == "-translated.gff")
            $gene_ann = $file;
    }

    //
    // Attempt to sort linkage groups numerically.
    //
    $lg_keys = array();

    foreach ($lgs as $lg) {
        $n = "";
        for ($i = 0; $i < strlen($lg); $i++) {
            if (ctype_digit($lg[$i]))
                $n .= $lg[$i];
        }
        $n = (strlen($n) > 0) ? $n : $lg;

        $lg_keys[$lg] = $n;
    }
    asort($lg_keys);
  
    print
        "<div class=\"body\">\n" .
        "<h1 class=\"head\">Chromonomer: $ver</h1>\n" .
        "<div class=\"version\">\n" .
        "<div class=\"chrrdesc\">";

    readfile("$file_path/$ver/chromonomer_description.log");
    
    print
        "</div>\n" .
        "<h3>View the assembly logs:</h3>\n" .
        "<table class=\"logs\">\n" .
        "<tr>\n";

    $i = 1;
    foreach ($lg_keys as $lg => $key) {
        $enc = urlencode($lg);
        print "<td><a href=\"lg.php?lg=$enc&v=$ver\">$lg</a></td>\n";

        if ($i % 8 == 0)
            print 
                "</tr>\n" .
                "<tr>\n";
        $i++;
    }
    while ($i % 8 != 1) {
        print "<td></td>\n";
        $i++;
    }

    print 
        "</tr>\n" .
        "</table>\n" .
        "<h3>Assembly Statistics:</h3>\n" .
        "<div class=\"summary\">\n";

    readfile("$file_path/$ver/chromonomer_summary.log");

    print
        "</div>\n" .
        "<h3>AGP Genome Assembly Descriptions</h3>\n" .
        "<ul>\n" .
        "  <li><a href=\"$root_path/$ver/$scaffold_agp\">AGP Scaffold Descriptions</a></li>\n" .
        "  <li><a href=\"$root_path/$ver/$genome_agp\">AGP Linkage Group Descriptions</a></li>\n";

    if (strlen($fasta) > 0)
        print "  <li><a href=\"$root_path/$ver/$fasta\">FASTA Integrated sequence</a></li>\n";
    if (strlen($gene_ann) > 0)
        print "  <li><a href=\"$root_path/$ver/$gene_ann\">Translated Gene Annotations</a></li>\n";

    print
        "</ul>\n" .
        "<h3>Problem Scaffolds</h3>\n" .
        "<table class=\"logs\">\n" .
        "<tr>\n" .
        "  <th>Scaffold</th>\n" . 
        "  <th>LG</th>\n" .
        "  <th><acronym title=\"Chromonomer attempted to split this scaffold and failed.\">Split Failed</acronym></th>\n" . 
        "  <th>Conflicted Markers</th>\n" . 
        "  <th><acronym title=\"Number of times this scaffold was split.\">Split Cnt</acronym></th>\n" .
        "<tr>\n";

    $fh = fopen("$file_path/$ver/problem_scaffolds.tsv", "r");

    if ($fh) {
        while (($buffer = fgets($fh, 4096)) !== false) {
            if ($buffer[0] == "#")
                continue;
            $parts = explode("\t", $buffer);

            $scaf = urlencode($parts[0]);
            $v    = urlencode($ver);

            if ($parts[2] == "true")
                $split = "<span class=\"bad\">$parts[2]</span>";
            else
                $split = "<span class=\"good\">$parts[2]</span>";

            print
                "<tr>\n" .
                "  <td><a href=\"$root_path/view_log.php?v=$v&log=$scaf\">$parts[0]</a></td>\n" .
                "  <td>$parts[1]</td>\n" .
                "  <td>$split</td>\n" .
                "  <td>$parts[3]</td>\n" .
                "  <td>$parts[4]</td>\n" .
                "</tr>\n";
        }
    }

    print
        "</table>\n" .
        "</div>\n";
}

write_footer();
?>
