// -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
//
// Copyright 2016, Julian Catchen <jcatchen@illinois.edu>
//
// This file is part of Chromonomer.
//
// Chromonomer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Chromonomer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Chromonomer.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef __LINKAGE_MAP_H__
#define __LINKAGE_MAP_H__

#include <utility>
using std::pair;
using std::make_pair;
#include <string>
using std::string;
#include <map>
using std::map;
#include <set>
using std::set;

#include "constants.h"
#include "scaffold.h"

class Node {
public:
    double cM;
    set<double> cMs;
    map<string, Scaffold> scaffolds;

    Node() {
	this->cM = 0.0;
    }
    Node(double cM) {
	this->cM = cM;
	this->cMs.insert(cM);
    }
    string id() const {
	char    s[id_len];
	string id;

	set<double>::iterator i = this->cMs.begin();
	
	do { 
	    snprintf(s, id_len, "%0.3f", *i);
	    s[id_len - 1] = '\0';
	    id += string(s);
	    i++;
	    if (i != this->cMs.end())
		id += "|";
	} while (i != this->cMs.end());

	return id;
    }
    bool operator ==(const Node &n) {
	return this->id() == n.id();
    }
};

#endif // __LINKAGE_MAP_H__
